@@ -108,6 +108,7 @@ pango_font_description_set_family (PangoFontDescription *desc,
     {
       desc->family_name = g_strdup (family);
       desc->mask |= PANGO_FONT_MASK_FAMILY;
+      desc->static_family = FALSE;
     }
   else
     {
@@ -160,7 +161,7 @@ pango_font_description_set_family_static (PangoFontDescription *desc,
  * Gets the family name field of a font description. See
  * pango_font_description_set_family().
  * 
- * Return value: The family name field, or %NULL if the family name is unset.
+ * Return value: The family name field. (Will be %NULL if not previously set.)
  **/
 G_CONST_RETURN char *
 pango_font_description_get_family (const PangoFontDescription *desc)
@@ -201,9 +202,8 @@ pango_font_description_set_style (PangoFontDescription *desc,
  * Gets the style field of a #PangoFontDescription. See
  * pango_font_description_set_style().
  * 
- * Return value: the style field for the font description. Returns
- *   %PANGO_STYLE_NORMAL if the style field is unset. Use
- *   pango_font_description_get_set_fields() to find out if
+ * Return value: the style field for the font description. 
+ *   Use pango_font_description_get_set_fields() to find out if
  *   the field was explicitely set or not.
  **/
 PangoStyle
@@ -239,8 +239,7 @@ pango_font_description_set_variant (PangoFontDescription *desc,
  * Gets the variant field of a #PangoFontDescription. See
  * pango_font_description_set_variant().
  * 
- * Return value: the variant field for the font description. Returns
- *   %PANGO_VARIANT_NORMAL if the variant field is unset. Use
+ * Return value: the variant field for the font description. Use
  *   pango_font_description_get_set_fields() to find out if
  *   the field was explicitely set or not.
  **/
@@ -279,8 +278,7 @@ pango_font_description_set_weight (PangoFontDescription *desc,
  * Gets the weight field of a font description. See
  * pango_font_description_set_weight().
  * 
- * Return value: the weight field for the font description. Returns
- *   %PANGO_WEIGHT_NORMAL if the weight field is unset. Use
+ * Return value: the weight field for the font description. Use
  *   pango_font_description_get_set_fields() to find out if
  *   the field was explicitely set or not.
  **/
@@ -317,8 +315,7 @@ pango_font_description_set_stretch (PangoFontDescription *desc,
  * Gets the stretch field of a font description.
  * See pango_font_description_set_stretch().
  * 
- * Return value: the stretch field for the font description. Returns
- *   %PANGO_STRETCH_NORMAL if the stretch field is unset. Use
+ * Return value: the stretch field for the font description. Use
  *   pango_font_description_get_set_fields() to find out if
  *   the field was explicitely set or not.
  **/
@@ -356,8 +353,8 @@ pango_font_description_set_size (PangoFontDescription *desc,
  * See pango_font_description_get_size().
  * 
  * Return value: the size field for the font description in pango
- *   units.  (PANGO_SCALE pango units equals one point.)  Returns 0 if
- *   the stretch field is unset. Use
+ *   units.  (PANGO_SCALE pango units equals one point). Returns 0 if
+ *   the stretch field has not previously been set.
  *   pango_font_description_get_set_fields() to find out if the field
  *   was explicitely set or not.
  **/
@@ -391,7 +388,12 @@ pango_font_description_get_set_fields (const PangoFontDescription *desc)
  * @desc: a #PangoFontDescription
  * @to_unset: bitmask of fields in the @desc to unset.
  * 
- * Unsets some of the fields in a #PangoFontDescription.
+ * Unsets some of the fields in a #PangoFontDescription. Note that
+ * this merely marks the fields cleared, it does not clear the
+ * settings for those fields, to clear a family name set with
+ * pango_font_description_set_family_static() so that it won't
+ * be returned by subsequent calls to pango_font_description_get_family(),
+ * you must actually call pango_font_description-set_family (desc, NULL);
  **/
 void
 pango_font_description_unset_fields (PangoFontDescription *desc,
@@ -399,22 +401,6 @@ pango_font_description_unset_fields (PangoFontDescription *desc,
 {
   g_return_if_fail (desc != NULL);
 
-  /* We reset cleared mask values back to defaults, to avoid
-   * having to check the mask for getters.
-   */
-  if (to_unset & PANGO_FONT_MASK_FAMILY)
-    pango_font_description_set_family (desc, NULL);
-  if (to_unset & PANGO_FONT_MASK_STYLE)
-    desc->style = PANGO_STYLE_NORMAL;
-  if (to_unset & PANGO_FONT_MASK_VARIANT)
-    desc->variant = PANGO_VARIANT_NORMAL;
-  if (to_unset & PANGO_FONT_MASK_WEIGHT)
-    desc->weight = PANGO_WEIGHT_NORMAL;
-  if (to_unset & PANGO_FONT_MASK_STRETCH)
-    desc->stretch = PANGO_STRETCH_NORMAL;
-  if (to_unset & PANGO_FONT_MASK_SIZE)
-    desc->size = 0;
-    
   desc->mask &= ~to_unset;
 }
 
@@ -570,6 +556,7 @@ pango_font_description_copy  (const PangoFontDescription  *desc)
   *result = *desc;
 
   result->family_name = g_strdup (result->family_name);
+  result->static_family = FALSE;
 
   return result;
 }
@@ -874,7 +861,10 @@ pango_font_description_from_string (const char *str)
     str++;
 
   if (str != last)
-    desc->family_name = g_strndup (str, last - str);
+    {
+      desc->family_name = g_strndup (str, last - str);
+      desc->mask |= PANGO_FONT_MASK_FAMILY;      
+    }
 
   return desc;
 }
@@ -899,7 +889,7 @@ append_field (GString *str, FieldMap *map, int n_elements, int val)
   
   if (str->len > 0 || str->str[str->len -1] != ' ')
     g_string_append_c (str, ' ');
-  g_string_sprintfa (str, "%d", val);
+  g_string_printfa (str, "%d", val);
 }
 
 /**
@@ -947,7 +937,7 @@ pango_font_description_to_string (const PangoFontDescription  *desc)
 
       /* FIXME: %g is not right here. We need to always use '.
        */
-      g_string_sprintfa (result, "%g", (double)desc->size / PANGO_SCALE);
+      g_string_printfa (result, "%g", (double)desc->size / PANGO_SCALE);
     }
   
   str = result->str;
@@ -1121,6 +1111,20 @@ pango_font_get_metrics (PangoFont        *font,
   return PANGO_FONT_GET_CLASS (font)->get_metrics (font, language);
 }
 
+GType
+pango_font_metrics_get_type (void)
+{
+  static GType our_type = 0;
+  
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("PangoFontMetrics",
+					     (GBoxedCopyFunc)pango_font_metrics_ref,
+					     (GBoxedFreeFunc)pango_font_metrics_unref);
+
+  return our_type;
+}
+
+
 /**
  * pango_font_metrics_new:
  * 
