@@ -70,8 +70,8 @@ pango_font_map_load_font  (PangoFontMap               *fontmap,
 /**
  * pango_font_map_list_families:
  * @fontmap: a #PangoFontMap
- * @families: location to store a pointer to an array of strings.
- *            This array should be freed with pango_font_map_free_families().
+ * @families: location to store a pointer to an array of PangoFontFamily *.
+ *            This array should be freed with g_free().
  * @n_families: location to store the number of elements in @descs
  * 
  * List all families for a fontmap. 
@@ -85,26 +85,3 @@ pango_font_map_list_families (PangoFontMap      *fontmap,
 
   PANGO_FONT_MAP_GET_CLASS (fontmap)->list_families (fontmap, families, n_families);
 }
-
-/**
- * pango_font_map_free_families:
- * @families: a list of families
- * @n_families: number of elements in @families
- * 
- * Free a list of families returned from pango_font_map_list_families()
- **/
-void
-pango_font_map_free_families (gchar       **families,
-			      int           n_families)
-{
-  int i;
-  
-  g_return_if_fail (n_families == 0 || families != NULL);
-
-  for (i=0; i<n_families; i++)
-    g_free (families[i]);
-
-  g_free (families);
-}
-
-
