@@ -574,18 +574,16 @@ read_config ()
       const char *envvar;
       
       config_hash = g_hash_table_new (g_str_hash, g_str_equal);
-      filename = g_strconcat (pango_get_sysconf_subdirectory (),
-			      G_DIR_SEPARATOR_S "pangorc",
-			      NULL);
+      filename = g_build_filename (pango_get_sysconf_subdirectory (),
+				   "pangorc",
+				   NULL);
       read_config_file (filename, FALSE);
       g_free (filename);
 
       home = g_get_home_dir ();
       if (home && *home)
 	{
-	  filename = g_strconcat (home,
-				  G_DIR_SEPARATOR_S ".pangorc",
-				  NULL);
+	  filename = g_build_filename (home, ".pangorc", NULL);
 	  read_config_file (filename, FALSE);
 	  g_free (filename);
 	}
@@ -650,9 +648,9 @@ pango_get_lib_subdirectory (void)
 }
 
 gboolean
-pango_parse_style (const char           *str,
-		   PangoFontDescription *desc,
-		   gboolean		 warn)
+pango_parse_style (const char *str,
+		   PangoStyle *style,
+		   gboolean    warn)
 {
   if (*str == '\0')
     return FALSE;
@@ -663,21 +661,21 @@ pango_parse_style (const char           *str,
     case 'N':
       if (g_strcasecmp (str, "normal") == 0)
 	{
-	  desc->style = PANGO_STYLE_NORMAL;
+	  *style = PANGO_STYLE_NORMAL;
 	  return TRUE;
 	}
       break;
     case 'i':
       if (g_strcasecmp (str, "italic") == 0)
 	{
-	  desc->style = PANGO_STYLE_ITALIC;
+	  *style = PANGO_STYLE_ITALIC;
 	  return TRUE;
 	}
       break;
     case 'o':
       if (g_strcasecmp (str, "oblique") == 0)
 	{
-	  desc->style = PANGO_STYLE_OBLIQUE;
+	  *style = PANGO_STYLE_OBLIQUE;
 	  return TRUE;
 	}
       break;
@@ -689,9 +687,9 @@ pango_parse_style (const char           *str,
 }
 
 gboolean
-pango_parse_variant (const char           *str,
-		     PangoFontDescription *desc,
-		     gboolean		   warn)
+pango_parse_variant (const char   *str,
+		     PangoVariant *variant,
+		     gboolean	   warn)
 {
   if (*str == '\0')
     return FALSE;
@@ -702,7 +700,7 @@ pango_parse_variant (const char           *str,
     case 'N':
       if (g_strcasecmp (str, "normal") == 0)
 	{
-	  desc->variant = PANGO_VARIANT_NORMAL;
+	  *variant = PANGO_VARIANT_NORMAL;
 	  return TRUE;
 	}
       break;
@@ -711,7 +709,7 @@ pango_parse_variant (const char           *str,
       if (g_strcasecmp (str, "small_caps") == 0 ||
 	  g_strcasecmp (str, "smallcaps") == 0)
 	{
-	  desc->variant = PANGO_VARIANT_SMALL_CAPS;
+	  *variant = PANGO_VARIANT_SMALL_CAPS;
 	  return TRUE;
 	}
       break;
@@ -723,9 +721,9 @@ pango_parse_variant (const char           *str,
 }
 
 gboolean
-pango_parse_weight (const char           *str,
-		    PangoFontDescription *desc,
-		    gboolean		  warn)
+pango_parse_weight (const char  *str,
+		    PangoWeight *weight,
+		    gboolean     warn)
 {
   if (*str == '\0')
     return FALSE;
@@ -736,7 +734,7 @@ pango_parse_weight (const char           *str,
     case 'B':
       if (g_strcasecmp (str, "bold") == 0)
 	{
-	  desc->weight = PANGO_WEIGHT_BOLD;
+	  *weight = PANGO_WEIGHT_BOLD;
 	  return TRUE;
 	}
       break;
@@ -744,7 +742,7 @@ pango_parse_weight (const char           *str,
     case 'H':
       if (g_strcasecmp (str, "heavy") == 0)
 	{
-	  desc->weight = PANGO_WEIGHT_HEAVY;
+	  *weight = PANGO_WEIGHT_HEAVY;
 	  return TRUE;
 	}
       break;
@@ -752,7 +750,7 @@ pango_parse_weight (const char           *str,
     case 'L':
       if (g_strcasecmp (str, "light") == 0)
 	{
-	  desc->weight = PANGO_WEIGHT_LIGHT;
+	  *weight = PANGO_WEIGHT_LIGHT;
 	  return TRUE;
 	}
       break;
@@ -760,7 +758,7 @@ pango_parse_weight (const char           *str,
     case 'N':
       if (g_strcasecmp (str, "normal") == 0)
 	{
-	  desc->weight = PANGO_WEIGHT_NORMAL;
+	  *weight = PANGO_WEIGHT_NORMAL;
 	  return TRUE;
 	}
       break;
@@ -768,12 +766,12 @@ pango_parse_weight (const char           *str,
     case 'U':
       if (g_strcasecmp (str, "ultralight") == 0)
 	{
-	  desc->weight = PANGO_WEIGHT_ULTRALIGHT;
+	  *weight = PANGO_WEIGHT_ULTRALIGHT;
 	  return TRUE;
 	}
       else if (g_strcasecmp (str, "ultrabold") == 0)
 	{
-	  desc->weight = PANGO_WEIGHT_ULTRABOLD;
+	  *weight = PANGO_WEIGHT_ULTRABOLD;
 	  return TRUE;
 	}
       break;
@@ -790,7 +788,7 @@ pango_parse_weight (const char           *str,
       {
 	char *end;
 
-	desc->weight = strtol (str, &end, 0);
+	*weight = strtol (str, &end, 0);
 	if (*end != '\0')
 	  {
 	    if (warn)
@@ -807,9 +805,9 @@ pango_parse_weight (const char           *str,
 }
 
 gboolean
-pango_parse_stretch (const char            *str,
-		     PangoFontDescription *desc,
-		     gboolean		   warn)
+pango_parse_stretch (const char   *str,
+		     PangoStretch *stretch,
+		     gboolean	   warn)
 {
   if (*str == '\0')
     return FALSE;
@@ -820,7 +818,7 @@ pango_parse_stretch (const char            *str,
     case 'C':
       if (g_strcasecmp (str, "condensed") == 0)
 	{
-	  desc->stretch = PANGO_STRETCH_CONDENSED;
+	  *stretch = PANGO_STRETCH_CONDENSED;
 	  return TRUE;
 	}
       break;
@@ -829,18 +827,18 @@ pango_parse_stretch (const char            *str,
       if (g_strcasecmp (str, "extra_condensed") == 0 ||
 	  g_strcasecmp (str, "extracondensed") == 0)
 	{
-	  desc->stretch = PANGO_STRETCH_EXTRA_CONDENSED;
+	  *stretch = PANGO_STRETCH_EXTRA_CONDENSED;
 	  return TRUE;
 	}
      if (g_strcasecmp (str, "extra_expanded") == 0 ||
 	 g_strcasecmp (str, "extraexpanded") == 0)
 	{
-	  desc->stretch = PANGO_STRETCH_EXTRA_EXPANDED;
+	  *stretch = PANGO_STRETCH_EXTRA_EXPANDED;
 	  return TRUE;
 	}
       if (g_strcasecmp (str, "expanded") == 0)
 	{
-	  desc->stretch = PANGO_STRETCH_EXPANDED;
+	  *stretch = PANGO_STRETCH_EXPANDED;
 	  return TRUE;
 	}
       break;
@@ -848,7 +846,7 @@ pango_parse_stretch (const char            *str,
     case 'N':
       if (g_strcasecmp (str, "normal") == 0)
 	{
-	  desc->stretch = PANGO_STRETCH_NORMAL;
+	  *stretch = PANGO_STRETCH_NORMAL;
 	  return TRUE;
 	}
       break;
@@ -857,13 +855,13 @@ pango_parse_stretch (const char            *str,
       if (g_strcasecmp (str, "semi_condensed") == 0 ||
 	  g_strcasecmp (str, "semicondensed") == 0)
 	{
-	  desc->stretch = PANGO_STRETCH_SEMI_CONDENSED;
+	  *stretch = PANGO_STRETCH_SEMI_CONDENSED;
 	  return TRUE;
 	}
       if (g_strcasecmp (str, "semi_expanded") == 0 ||
 	  g_strcasecmp (str, "semiexpanded") == 0)
 	{
-	  desc->stretch = PANGO_STRETCH_SEMI_EXPANDED;
+	  *stretch = PANGO_STRETCH_SEMI_EXPANDED;
 	  return TRUE;
 	}
       break;
@@ -872,13 +870,13 @@ pango_parse_stretch (const char            *str,
       if (g_strcasecmp (str, "ultra_condensed") == 0 ||
 	  g_strcasecmp (str, "ultracondensed") == 0)
 	{
-	  desc->stretch = PANGO_STRETCH_ULTRA_CONDENSED;
+	  *stretch = PANGO_STRETCH_ULTRA_CONDENSED;
 	  return TRUE;
 	}
       if (g_strcasecmp (str, "ultra_expanded") == 0 ||
 	  g_strcasecmp (str, "ultraexpanded") == 0)
 	{
-	  desc->variant = PANGO_STRETCH_ULTRA_EXPANDED;
+	  *stretch = PANGO_STRETCH_ULTRA_EXPANDED;
 	  return TRUE;
 	}
       break;
@@ -952,10 +950,8 @@ pango_language_get_type (void)
   
   if (our_type == 0)
     our_type = g_boxed_type_register_static ("PangoLanguage",
-                                             NULL,
                                              (GBoxedCopyFunc)pango_language_copy,
-                                             (GBoxedFreeFunc)pango_language_free,
-                                             FALSE);
+                                             (GBoxedFreeFunc)pango_language_free);
   return our_type;
 }
 
@@ -1020,6 +1016,8 @@ pango_language_from_string (const char *language)
  * in the list if the range is '*', the range is exactly the tag,
  * or the range is a prefix of the tag, and the character after the
  * tag is '-'.
+ *
+ * Returns: %TRUE if a match was found.
  **/
 gboolean
 pango_language_matches (PangoLanguage *language,
