@@ -455,8 +455,8 @@ pango_win32_font_describe (PangoFont *font)
   PangoFontDescription *desc;
   PangoWin32Font *win32font = PANGO_WIN32_FONT (font);
 
-  desc = pango_font_description_copy (&win32font->entry->description);
-  desc->size = win32font->size;
+  desc = pango_font_description_copy (win32font->entry->description);
+  pango_font_description_set_size (desc, win32font->size);
   
   return desc;
 }
