@@ -41,22 +41,21 @@ static PangoFontClass *parent_class;	/* Parent class structure for PangoXftFont
 
 static void pango_xft_font_class_init (PangoXftFontClass *class);
 static void pango_xft_font_init       (PangoXftFont      *xfont);
-static void pango_xft_font_shutdown   (GObject         *object);
+static void pango_xft_font_dispose    (GObject         *object);
 static void pango_xft_font_finalize   (GObject         *object);
 
 static PangoFontDescription *pango_xft_font_describe          (PangoFont        *font);
 static PangoCoverage *       pango_xft_font_get_coverage      (PangoFont        *font,
-							       const char       *lang);
+							       PangoLanguage    *language);
 static PangoEngineShape *    pango_xft_font_find_shaper       (PangoFont        *font,
-							       const char       *lang,
+							       PangoLanguage    *language,
 							       guint32           ch);
 static void                  pango_xft_font_get_glyph_extents (PangoFont        *font,
 							       PangoGlyph        glyph,
 							       PangoRectangle   *ink_rect,
 							       PangoRectangle   *logical_rect);
-static void                  pango_xft_font_get_metrics       (PangoFont        *font,
-							       const gchar      *lang,
-							       PangoFontMetrics *metrics);
+static PangoFontMetrics *    pango_xft_font_get_metrics       (PangoFont        *font,
+							       PangoLanguage    *language);
 
 
 GType
@@ -102,7 +101,7 @@ pango_xft_font_class_init (PangoXftFontClass *class)
   parent_class = g_type_class_peek_parent (class);
   
   object_class->finalize = pango_xft_font_finalize;
-  object_class->shutdown = pango_xft_font_shutdown;
+  object_class->dispose = pango_xft_font_dispose;
   
   font_class->describe = pango_xft_font_describe;
   font_class->get_coverage = pango_xft_font_get_coverage;
@@ -143,7 +142,7 @@ get_mini_font (PangoFont *font)
   if (!xfont->mini_font)
     {
       Display *display;
-      PangoFontDescription desc;
+      PangoFontDescription *desc = pango_font_description_new ();
       int i;
       int width = 0, height = 0;
       XGlyphInfo extents;
@@ -151,15 +150,14 @@ get_mini_font (PangoFont *font)
       FT_Face face;
       
       _pango_xft_font_map_get_info (xfont->fontmap, &display, NULL);
+
+      pango_font_description_set_family_static (desc, "monospace");
+      pango_font_description_set_size (desc,
+				       0.5 * pango_font_description_get_size (xfont->description));
       
-      desc.family_name = "monospace";
-      desc.style = PANGO_STYLE_NORMAL;
-      desc.variant = PANGO_VARIANT_NORMAL;
-      desc.weight = PANGO_WEIGHT_NORMAL;
+      xfont->mini_font = pango_font_map_load_font (xfont->fontmap, desc);
+      pango_font_description_free (desc);
       
-      desc.size = 0.5 * xfont->description->size;
-      
-      xfont->mini_font = pango_font_map_load_font (xfont->fontmap, &desc);
       mini_xft = ((PangoXftFont *)xfont->mini_font)->xft_font;
       face = pango_xft_font_get_face (xfont->mini_font);
       
@@ -280,23 +278,23 @@ pango_xft_render (XftDraw          *draw,
     }
 }
 
-static void
+static PangoFontMetrics *
 pango_xft_font_get_metrics (PangoFont        *font,
-			    const gchar      *lang,
-			    PangoFontMetrics *metrics)
+			    PangoLanguage    *language)
 {
   PangoXftFont *xfont = (PangoXftFont *)font;
+  PangoFontMetrics *metrics = pango_font_metrics_new ();
 
   metrics->ascent = PANGO_SCALE * xfont->xft_font->ascent;
   metrics->descent = PANGO_SCALE * xfont->xft_font->descent;
   metrics->approximate_digit_width = PANGO_SCALE * xfont->xft_font->max_advance_width;
   metrics->approximate_char_width = PANGO_SCALE * xfont->xft_font->max_advance_width;
-  
-  return;
+
+  return metrics;
 }
 
 static void
-pango_xft_font_shutdown (GObject *object)
+pango_xft_font_dispose (GObject *object)
 {
   PangoXftFont *xfont = PANGO_XFT_FONT (object);
 
@@ -307,13 +305,16 @@ pango_xft_font_shutdown (GObject *object)
   if (!xfont->in_cache && xfont->fontmap)
     _pango_xft_font_map_cache_add (xfont->fontmap, xfont);
 
-  G_OBJECT_CLASS (parent_class)->shutdown (object);
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
 pango_xft_font_finalize (GObject *object)
 {
   PangoXftFont *xfont = (PangoXftFont *)object;
+  Display *display;
+  
+  _pango_xft_font_map_get_info (xfont->fontmap, &display, NULL);
 
   _pango_xft_font_map_remove (xfont->fontmap, xfont);
 
@@ -323,6 +324,10 @@ pango_xft_font_finalize (GObject *object)
   if (xfont->ot_info)
     g_object_unref (xfont->ot_info);
 
+  pango_font_description_free (xfont->description);
+  
+  XftFontClose (display, xfont->xft_font);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -335,10 +340,11 @@ pango_xft_font_describe (PangoFont *font)
 }
 
 static PangoCoverage *
-pango_xft_font_get_coverage (PangoFont  *font,
-			     const char *lang)
+pango_xft_font_get_coverage (PangoFont     *font,
+			     PangoLanguage *language)
 {
   PangoXftFont *xfont = (PangoXftFont *)font;
+  const gchar *family = pango_font_description_get_family (xfont->description);
   FT_Face face;
   PangoCoverage *coverage;
   Display *display;
@@ -346,8 +352,7 @@ pango_xft_font_get_coverage (PangoFont  *font,
   
   _pango_xft_font_map_get_info (xfont->fontmap, &display, NULL);
   
-  coverage = _pango_xft_font_map_get_coverage (xfont->fontmap,
-					       xfont->description->family_name);
+  coverage = _pango_xft_font_map_get_coverage (xfont->fontmap, family);
 
   if (coverage)
     return pango_coverage_ref (coverage);
@@ -365,7 +370,7 @@ pango_xft_font_get_coverage (PangoFont  *font,
 	pango_coverage_set (coverage, i, PANGO_COVERAGE_EXACT);
     }
 
-  _pango_xft_font_map_set_coverage (xfont->fontmap, xfont->description->family_name, coverage);
+  _pango_xft_font_map_set_coverage (xfont->fontmap, family, coverage);
       
   return coverage;
 }
@@ -428,7 +433,7 @@ pango_xft_font_get_glyph_extents (PangoFont        *font,
 }
 
 static PangoMap *
-pango_xft_get_shaper_map (const char *lang)
+pango_xft_get_shaper_map (PangoLanguage *language)
 {
   static guint engine_type_id = 0;
   static guint render_type_id = 0;
@@ -439,17 +444,17 @@ pango_xft_get_shaper_map (const char *lang)
       render_type_id = g_quark_from_static_string (PANGO_RENDER_TYPE_XFT);
     }
   
-  return pango_find_map (lang, engine_type_id, render_type_id);
+  return pango_find_map (language, engine_type_id, render_type_id);
 }
 
 static PangoEngineShape *
-pango_xft_font_find_shaper (PangoFont   *font,
-			  const gchar *lang,
-			  guint32      ch)
+pango_xft_font_find_shaper (PangoFont     *font,
+			    PangoLanguage *language,
+			    guint32        ch)
 {
   PangoMap *shape_map = NULL;
 
-  shape_map = pango_xft_get_shaper_map (lang);
+  shape_map = pango_xft_get_shaper_map (language);
   return (PangoEngineShape *)pango_map_get_engine (shape_map, ch);
 }
 
