@@ -19,7 +19,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <pango/pango.h>
+#include <pango/pango-glyph.h>
+#include <pango/pango-engine.h>
 
 /**
  * pango_shape:
@@ -52,4 +53,6 @@ void pango_shape (const gchar      *text,
       
       glyphs->log_clusters[0] = 0;
     }
+
+  g_assert (glyphs->num_glyphs > 0);
 }
